@@ -217,6 +217,10 @@ sub _init {
         Data::Printer::Common::_fetch_scalar_or_default($props, 'multiline', 1)
     );
 
+    $self->fulldump(
+        Data::Printer::Common::_fetch_scalar_or_default($props, 'fulldump', 0)
+    );
+
     $self->_load_colors($props);
     $self->_load_filters($props);
     $self->output($props->{output} || 'stderr');
@@ -307,6 +311,29 @@ sub multiline {
     return $self->{multiline};
 }
 
+sub fulldump {
+    my ($self, $value) = @_;
+    if (defined $value) {
+        $self->{fulldump} = !!$value;
+        if ($value) {
+            $self->{_original_max_string} = $self->max_string;
+            $self->max_string(0);
+            $self->{_original_max_array} = $self->max_array;
+            $self->max_array(0);
+            $self->{_original_max_hash} = $self->max_hash;
+            $self->max_hash(0);
+        }
+        else {
+            $self->max_string($self->{_original_max_string})
+                if exists $self->{_original_max_string};
+            $self->max_array($self->{_original_max_array})
+                if exists $self->{_original_max_array};
+            $self->max_hash($self->{_original_max_hash})
+                if exists $self->{_original_max_hash};
+        }
+    }
+}
+
 sub _load_filters {
     my ($self, $props) = @_;
 
@@ -913,6 +940,8 @@ Most of them are described in L<Data::Printer>.
 
 =item * multiline - set to 0 to disable linebreaks, disable index and use ':' as hash separator.
 
+=item * fulldump - set to 1 to disable mas_string, max_array and max_hash at the same time.
+
 =item * output - where the user wants the output to be printed. Defaults to 'stderr', could be 'stdout', \$string or $filehandle.
 
 =item * output_handle - stores the proper handle for the given output so you can print to it.
